/*----------------------------------------------
Filename: NetsTensor.c

by Wenyuan Li, February 2009
----------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include <limits.h>
#include <sys/timeb.h>
#include "my_getopt-1.5/getopt.h"
#include "UtilsOfTensor.h"
#include "Tensor_Private.h"
#include "Tensor_LpRelax.h"

/*-----------------------------------
command line parser's data structure
-----------------------------------*/
struct option longopts[] =
{
	/* name,                      has_arg,          flag, val */ /* longind */
	{ "help",                     no_argument,       0,   'h' }, /*       0 */
	{ "maxPattern",               required_argument, 0,   'p' }, /*       1 */
	{ "nIteration",               required_argument, 0,   'i' }, /*       2 */
	{ "nStage",                   required_argument, 0,   's' }, /*       3 */
	{ "minNode",                  required_argument, 0,   'g' }, /*       4 */
	{ "maxNode",                  required_argument, 0,   'G' }, /*       4 */
	{ "minNet",	                  required_argument, 0,   'n' }, /*       5 */
	{ "minDensity",               required_argument, 0,   'd' }, /*       6 */
	{ "maskStrategy",             required_argument, 0,   'S' }, /*       7 */
	{ "overlapPatternChoose",     required_argument, 0,   'O' }, /*       8 */
	{ "prefixResultFilename",     required_argument, 0,   'f' }, /*       9 */
	{ "selectedDatasetsListFile", required_argument, 0,   'D' }, /*       10*/
	{ "networksPath",             required_argument, 0,   'N' }, /*       11*/
	{ "resultPath",               required_argument, 0,   'R' }, /*       12*/
	{ "nEdgesLoad",               required_argument, 0,   'E' }, /*       13*/
	{ "resume",                   no_argument,       0,   'r' }, /*       14*/
	{ "loadUnweighted",           no_argument,       0,   'u' }, /*       15*/
	{ "rangeWeight",              required_argument, 0,   'w' }, /*       16*/
	{ "removeHeavyFrequentEdges", required_argument, 0,   'e' }, /*       17*/
	{ "Init_xy",                  required_argument, 0,   'I' }, /*       18*/
	{ "suffixdataFile",           required_argument, 0,   'F' }, /*       19*/
	{ "ParmsFile",                required_argument, 0,   'P' }, /*       19*/
	{ "ResultFile",               required_argument, 0,   'o' }, /*       19*/
	{ "succintOutput",            no_argument,       0,   'c' }, /*       15*/
	{ "muleInput",                no_argument,		 0,   'M' }, /*       19*/
	{ "muleOutput",               no_argument,		 0,   'm' }, /*       19*/
	{ "nodeTotalNumber",        required_argument, 0,   't' }, /*       19*/
	{ "excludeNodesIndexesFile",  required_argument, 0,   'x' }, /*       19*/
	{ "excludeEdgesOfDirectNeighborNodes",  no_argument, 0,   'X' }, /*       19*/
	/* end-of-list marker */
	{ 0, 0, 0, 0 }
};
char *shortopts = "hrucp:i:s:g:G:n:d:S:O:f:D:N:R:E:w:e:I:F:P:o:M:m:tx:X"; /* short options string */
int longind = 0; /* long option list index */

/*----------------------
 Global Variables
----------------------*/
LIST_STRING selectedNetsID; /* ID list of selected networks */
LIST_STRING selectedNetsAnnotation; /* annotation list of selected networks */
VEC_UINT selectedNets_INDEX;  /* index list of selected networks (start from 0) */
TENSOROFNETS t, t_backup; /* tensor of networks */
int nPattern_max=MAXPATTERN_DEFAULT;
unsigned int nIteration=NITERATION_DEFAULT; /* number of iterations in algorithm */
#ifdef LP_RELAX
	unsigned int nStage=NSTAGE_DEFAULT;  /* number of stages in algorithm */
#endif
unsigned int minGene=MINGENE_DEFAULT, minNet=MINNET_DEFAULT; /* minimal number of genes and nets in the pattern */
unsigned int maxGene = MAXGENE; /* maximum number of genes in the pattern */
double minDensity=MIN_DENSITY_DEFAULT; /* minimal density of frequent dense subgraphs */
enum MASK_STRATEGY mask_strategy_code;
char mask_strategy_name[MAXCHAR]=MASK_STRATEGY_NAME_DEFAULT;
char overlapPatternChoose[MAXCHAR]=OVERLAPPATTERNCHOOSE_DEFAULT;
PARMS_SETTING parms_setting; /* multiple sets of parameters */
unsigned int nEdgesLoad = NEDGES_LOAD_DEFAULT;
VEC_FLOAT rangeWeight; /* minimum and maximum weight of edges, that are allowd to be loaded to memory, and used in tensor analysis, when minmaxWeight.n=0, it means use "ALL" edges without considering weight's range */
unsigned int loadUnweighted = FALSE; /* load networks in unweighted or weighted manner. Default is to load networks with their edges' weights. */
VEC_FLOAT removeHeavyFrequentEdges; /* thresholds of weight and frequency of the edges that will not be counted in tensor analysis. It is given like "[0.6,0.4]", meaning "remove" edges whose weight>=0.6, and frequency>=%40 */
int howtoInit_xy=INIT_XY_DEFAULT; /* method of initializing vectors x and y: random vector or ones vector */
char path_nets[MAXCHAR] = PATH_NETS_DEFAULT;
char path_result[MAXCHAR] =  PATH_RESULT_DEFAULT;
char fn_results[MAXCHAR] = PREFIX_RESULTFILE_DEFAULT; /* output results file */
char fn_selectedDatasetsList[MAXCHAR] = ""; /* list of selected networks' IDs*/
char suffixdataFile[MAXCHAR] = SUFFIX_DATAFILE; /* suffix of network files, default is "SUFFIX_DATAFILE" */
char fn_parms[MAXCHAR]=""; /* the parms file */
unsigned int resume=RESUME_LASTRUN_DEFAULT; /* default value is RESUME_LASTRUN_DEFAULT (start a new run); tell whether resume the last run (set as 1), or start a new run (set as 0).*/
unsigned int geneTotalNumber=0;
char fn_excluded_genes[MAXCHAR]=""; /* the file containing a list of gene indexes which are exluded from the network analysis */
VEC_UINT genes_index_excluded; /* The indexes of those genes which are excluded in the network */
VEC_UINT binary_vec_genes_index_excluded; /* The binary vector of those genes which are excluded in the network. Its length is #gene. When a gene is excluded, its value is 1; otherwise, it is 0. */
unsigned int excludeEdgesOfDirectNeighborGenes=INCLUDE_EDGES_OF_DIRECT_NEIGHBORS;

void printmsg(FILE* stream, PARMS_SETTING p)
{
	fprintf( stream, "\nTensor Computation of Multiple Networks for Frequent Dense Subgraphs Discovery\n"  );
	fprintf( stream, "Tensor Method: Multi-Stage Convex Relaxation (MSCR)\n");
	fprintf( stream, "\nUsage: unweightedNetsTensor [options] ...\n"
		             "Options:\n"
					 "\t-h or --help      :show this message and exit\n"
					 "\t-pNUM or --maxPattern=NUM  :number of patterns to be discovered, default(unlimited: %d)\n"
					 "\t-iNUM or --nIteration=NUM  :number of iterations, default(%u)\n"
					 "\t-sNUM or --nStage=NUM      :number of stages, default(%u)\n"
					 "\t-tNUM or --nodeTotalNumber=NUM     :total number of nodes in the contracted graphs, REQUIRED\n"
					 "\t-gNUM or --minNode=NUM     :minimum number of nodes, default(%u)\n"
					 "\t-GNUM or --maxNode=NUM     :maximum number of nodes, default(%u)\n"
					 "\t-nNUM or --minNet=NUM      :minimum number of graphs, default(%u)\n"
					 "\t-dNUM or --minDensity=NUM  :minimum density (0<d<=1)  :default(%g)\n"
					 "\t-SSTRING or --maskStrategy=STRING  :%s,%s or %s (default: %s)\n"
					 "\t-OSTRING or --overlapPatternChoose=STRING  :%s,%s, %s or %s (default: %s)\n"
					 "\t-fSTRING or --prefixResultFilename=STRING  :default(%s)\n"
					 "\t-FSTRING or --suffixdataFile=STRING        :default(%s)\n"
					 "\t-DSTRING or --selectedDatasetsListFile=FILE  :required(put in the networks path)\n"
					 "\t-NSTRING or --networksPath=PATH :default(%s)\n"
					 "\t-RSTRING or --resultPath=PATH   :default(%s)\n"
					 "\t-ENUM or --nEdgesLoad=NUM   :max #edges loaded for each network, default(%u)\n"
					 "\t-r or --resume              :resume the last run\n"
					 "\t-c or --succintOutput       :output only PATTERN file"
					 "\t-u or --loadUnweighted      :load networks in the unweighted manner.\n\n"
					 "\t-wSTRING or --rangeWeight=STRING : minimum and maximum weight of edges that can be loaded to memory, default (""%s"")\n"
					 "\t                             could be like ""[0.5,0.8]""\n"
					 "\t-eSTRING or --removeHeavyFrequentEdges=STRING : threshold of weight and frequency of the edge that will not be\n"
					 "\t                                          counted in tensor analysis. If this option is not set, then do\n"
					 "\t                                          NOT remove heavy-frequency-edges; otherwise remove them before\n"
					 "\t                                          tensor analysis. It is given like ""[0.6,0.4]"", meaning ""remove""\n"
					 "\t                                          edges whose weight>=0.6, and frequency>=\%40, default (""%s"")\n"
					 "\t-INUM or --Init_xy=NUM : %u, use random values to initialize vectors x and y;\n"
					 "\t                         %u, use ones to initialize x and y;\n"
					 "\t                         %u, use random unit vector to initialize x and y;\n"
					 "\t                         default (""%u"")\n"
					 "\t-PSTRING or --ParmsFile=STRING  : parameters file. If not setting, use default parameters.\n"
					 "\t-oSTRING or --ResultFile=STRING : result filename(no path included). If not setting, use default filename(with params).\n"
					 "\t-xSTRING or --excludeNodesIndexesFile=STRING : filename(path must be included).\n"
					 "\t-X or --excludeEdgesOfDirectNeighborNodes    : if two nodes (domains) are adjacent in genome (their indexes are of difference 1), remove the edge connecting them.\n",
		    MAXPATTERN_DEFAULT,NITERATION_DEFAULT,NSTAGE_DEFAULT,MINGENE_DEFAULT,MAXGENE,MINNET_DEFAULT,MIN_DENSITY_DEFAULT,
		    MASK_STRATEGY_NAME_EDGES_PATTERN,MASK_STRATEGY_NAME_EDGES_ALLNETS,MASK_STRATEGY_NAME_GENES,MASK_STRATEGY_NAME_DEFAULT,
			OVERLAPPATTERNCHOOSE_MORE_NETS,OVERLAPPATTERNCHOOSE_MORE_GENES,OVERLAPPATTERNCHOOSE_BOTH,OVERLAPPATTERNCHOOSE_NONZEROS,OVERLAPPATTERNCHOOSE_DEFAULT,
			PREFIX_RESULTFILE_DEFAULT,SUFFIX_DATAFILE,PATH_NETS_DEFAULT,PATH_RESULT_DEFAULT,NEDGES_LOAD_DEFAULT,RANGE_WEIGHT_DEFAULT,REMOVE_HEAVYFREQUENTEDGES_DEFAULT,
			INIT_XY_BY_RAND, INIT_XY_BY_ONES, INIT_XY_BY_UNIT, INIT_XY_DEFAULT);
	fprintf( stream, "Algorithm predefined parameters, files and directories\n" );
	print_PARMS_SETTING(stream, p, "\t");
	fprintf( stream, "\tmaxNode is %u", MAXGENE );
	fprintf( stream, "\n\tdatasets Path - %s", PATH_NETS_DEFAULT);
	fprintf( stream, "\n\tdatasets filenames prefix and suffix - '%s' and '%s'", PREFIX_DATAFILE, SUFFIX_DATAFILE);
	fprintf( stream, "\n\tresults Path - %s\n\n", PATH_RESULT_DEFAULT);
	fprintf( stream, "Please email XJZHOU@usc.edu or WEL@usc.edu\n");
	fprintf( stream, "Further refers to http:\/\/zhoulab.usc.edu for help.\n\n");
}

void print_current_time( FILE* stream )
{
	time_t curtime;
	struct tm *loctime;
	curtime = time (NULL); /* Get the current time.  */
	loctime = localtime (&curtime); /* Convert it to local time representation.  */
	fputs (asctime (loctime), stream); /* Print out the date and time in the standard format.  */
}

void printparms(FILE* stream, PARMS_SETTING p)
{
	fprintf( stream, "#-----------------------------\n"  );
	fprintf( stream, "# Tensor Computation of Multiple Networks for Frequent Dense Subgraphs Discovery\n"  );
	fprintf( stream, "#------- Input Files -------\n"  );
	fprintf( stream, "# selectedDatasetsListFile: %s\n", fn_selectedDatasetsList );
	fprintf( stream, "# #Nodes: %u\n", geneTotalNumber );
	fprintf( stream, "# #Networks: %u\n", selectedNetsID.n );
	if (strlen(fn_excluded_genes)>0)
		fprintf( stream, "# excludeNodesIndexesFile: %s\n", fn_excluded_genes );
	if (excludeEdgesOfDirectNeighborGenes==EXCLUDE_EDGES_OF_DIRECT_NEIGHBORS)
		fprintf( stream, "# exclude edges whose two nodes (domains) are direct neighbors in genome sequence (their indexes are of difference 1)." );
	fprintf( stream, "# #max_Load_Edges_each_network: %u\n", nEdgesLoad );
	if (loadUnweighted)
		fprintf( stream, "# #loadUnweighted = TRUE (load networks in the unweighted manner)\n" );
	if (rangeWeight.v==NULL) fprintf( stream, "# #range_of_loaded_edges: %s\n", RANGE_WEIGHT_ALL );
	else fprintf( stream, "# #range_of_loaded_edges: [%g,%g]\n", rangeWeight.v[0], rangeWeight.v[1] );
	if (removeHeavyFrequentEdges.v==NULL) fprintf( stream, "# removeHeavyFrequentEdges: %s\n", REMOVE_HEAVYFREQUENTEDGES_NONE );
	else fprintf( stream, "# removeHeavyFrequentEdges: [%g, %g]\n", removeHeavyFrequentEdges.v[0], removeHeavyFrequentEdges.v[1] );
	fprintf( stream, "#----- Input Parameters ------\n"  );
	print_PARMS_SETTING(stream, p, "# ");
	fprintf( stream, "# #MAX_EXCHANGE_CONTINUOUS_PSEUDOPATTERN: %u\n", MAX_EXCHANGE_CONTINUOUS_PSEUDOPATTERN );
	fprintf( stream, "# #MAX_CONTINUOUS_PSEUDOPATTERN: %u\n", MAX_CONTINUOUS_PSEUDOPATTERN );
	fprintf( stream, "# #E_ZERO: %g\n", E_ZERO );
	fprintf( stream, "# #Stage= %u\n", nStage );
	if (nPattern_max==NPATTERN_UNLIMITED)
		fprintf( stream, "# #Pattern= unlimited (#patterns to be discovered)\n" );
	else
		fprintf( stream, "# #Pattern= %d (#patterns to be discovered)\n", nPattern_max );
	fprintf( stream, "# #Iteration= %u\n", nIteration );
	fprintf( stream, "# maskStrategy= %s\n", mask_strategy_name );
	fprintf( stream, "# overlapPatternChoose= %s\n", overlapPatternChoose );
	if (strcmp(overlapPatternChoose,OVERLAPPATTERNCHOOSE_NONZEROS)!=0) {
		fprintf( stream, "# minNode= %u (in pattern)\n", minGene );
		fprintf( stream, "# minNet= %u (in pattern)\n", minNet );
		fprintf( stream, "# minDensity= %g\n", minDensity );
	} else {
		fprintf( stream, "   indicating no ""minGene"",""minNet"",""minDensity"" are needed\n" );
	}
	fprintf( stream, "# maxNode= %u (in pattern)\n", maxGene );
	switch (howtoInit_xy) {
		case INIT_XY_BY_RAND:
			fprintf( stream, "# howtoInit_xy= random\n" );
			break;
		case INIT_XY_BY_ONES:
			fprintf( stream, "# howtoInit_xy= ones\n" );
			break;
		case INIT_XY_BY_UNIT:
			fprintf( stream, "# howtoInit_xy= random unit vector\n" );
			break;
		default:
			fprintf( stderr, "Error: howtoInit_xy is not %u, %u, or %u.\nExit.\n", INIT_XY_BY_ONES, INIT_XY_BY_RAND, INIT_XY_BY_UNIT );
			exit( -1 );
	}
	fprintf( stream, "#------- Output File --------\n"  );
	fprintf( stream, "# prefix of resultFile: %s\n", fn_results );
	if (succintOutput)
		fprintf( stream, "# succintOutput: TRUE - only output PATTERN and SETTING file\n" );
	fprintf( stream, "#-----------------------------\n\n"  );
}

void print_startmsg( FILE* stream )
{
	fprintf( stream, "Experiment starts at "  );
	print_current_time( stream );
	fprintf( stream, "\n");
}

void print_dataloadmsg( FILE* stream )
{
	fprintf( stream, "Data loaded at " );
	print_current_time( stream );
	fprintf( stream, "\n");
}

void print_resumemsg( FILE* stream )
{
	fprintf( stream, "Experiment resumes at " );
	print_current_time( stream );
	fprintf( stream, "\n");
}

void print_closingmsg( FILE* stream )
{
	fprintf( stream, "Experiment ends at "  );
	print_current_time( stream );
	fprintf( stream, "\n");
}

void parse_command_line(int argc, char * argv[])
{
	int opt; /* during argument parsing, opt contains the return value from getopt() */
	int longind = 0; /* long option list index */
	char rangeWeight_string[MAXCHAR]="", removeHeavyFrequentEdges_string[MAXCHAR]="";
	/* parse all options from the command line */
	while ((opt=getopt_long(argc, argv, shortopts, longopts, &longind)) != -1) {
		switch (opt) {
			case 'h': /* -help */
				printmsg( stdout, parms_setting ); /* 'parms_setting' is a global variable */
				exit(-1);
			case 'r': /* -resume */
				resume = 1;
				break;
			case 'c': /* -succintOutput */
				succintOutput = TRUE;
				break;
			case 'M': /* -muleInput */
				muleInput = TRUE;
				break;
			case 'm': /* -muleOutput */
				muleOutput = TRUE;
				break;
			case 't': /* -geneTotalNumber=NUM */
				geneTotalNumber = (unsigned int)atoi(optarg);
				break;
			case 'p': /* -maxPattern=NUM */
				nPattern_max = atoi(optarg);
				if (nPattern_max==NPATTERN_UNLIMITED)
					nPattern_max=INT_MAX;
				break;
			case 'i': /* -nIteration=NUM */
				nIteration = (unsigned int)atoi(optarg);
				break;
			case 's': /* -nStage=NUM */
				nStage = (unsigned int)atoi(optarg);
				break;
			case 'g': /* -minGene=NUM */
				minGene = (unsigned int)atoi(optarg);
				break;
			case 'G': /* -maxGene=NUM */
				maxGene = (unsigned int)atoi(optarg);
				break;
			case 'n': /* -minNet=NUM */
				minNet = (unsigned int)atoi(optarg);
				break;
			case 'd': /* -minDensity=NUM */
				minDensity = atof(optarg);
				break;
			case 'S': /* -maskStrategy=STRING */
				strcpy( mask_strategy_name, optarg );
				break;
			case 'O':  /* -overlapPatternChoose=STRING */
				strcpy( overlapPatternChoose, optarg );
				break;
			case 'f': /* -prefixResultFilename=STRING */
				strcpy( fn_results, optarg );
				useGivenResultFilename = TRUE;
				break;
			case 'F': /* -suffixdataFile=STRING */
				strcpy( suffixdataFile, optarg );
				break;
			case 'D': /* -selectedDatasetsListFile=FILE */
				strcpy( fn_selectedDatasetsList, optarg );
				break;
			case 'N': /* -networksPath=PATH */
				strcpy( path_nets, optarg );
				break;
			case 'R': /* -resultPath=PATH */
				strcpy( path_result, optarg );
				break;
			case 'E': /* -nEdgesLoad=NUM */
				nEdgesLoad = (unsigned int)atoi(optarg);
				break;
			case 'u': /* -loadUnweighted */
				loadUnweighted = TRUE;
				break;
			case 'w': /* -rangeWeight=STRING */
				strcpy( rangeWeight_string, optarg );
				break;
			case 'e': /* -removeHeavyFrequentEdges=STRING */
				strcpy( removeHeavyFrequentEdges_string, optarg );
				break;
			case 'I': /* -Init_xy=NUM */
				howtoInit_xy = atoi(optarg);
				break;
			case 'P': /* -ParmsFile=FILE */
				strcpy( fn_parms, optarg );
				break;
			case 'o': /* -ResultFile=FILE */
				strcpy( fn_results, optarg );
				useGivenResultFilename = TRUE;
				break;
			case 'x': /* -ResultFile=FILE */
				strcpy( fn_excluded_genes, optarg );
				break;
			case 'X': /* -excludeGenesDirectNeighbors */
				excludeEdgesOfDirectNeighborGenes = EXCLUDE_EDGES_OF_DIRECT_NEIGHBORS;
				break;
			default: /* something unexpected has happened */
				printmsg( stderr, parms_setting ); /* 'parms_setting' is a global variable */
				exit(-1);
		}
	}
	if (minGene<MINGENE_DEFAULT) { fprintf(stderr,"\nError: -minNode=NUM should be >=%u\n",MINGENE_DEFAULT); printmsg(stderr, parms_setting); exit(-1); }
	if (minDensity>1 || minDensity<=0) { fprintf(stderr,"\nError: -minDensity=NUM should be >=0 and <=1\n"); printmsg(stderr, parms_setting); exit(-1); }
	if (minNet<MINNET_DEFAULT) { fprintf(stderr,"\nError: -minNet=NUM should be >=%u\n",MINNET_DEFAULT); printmsg(stderr, parms_setting); exit(-1); }
	/* set default values, if they are not given in the command line */
	mask_strategy_code = checkname_MASK_STRATEGY( mask_strategy_name );
	if (mask_strategy_code==MASKSTRATEGY_ERROR) {
		fprintf(stderr, "\nError: -maskStrategy=STRING (%s) is not correct!\n", mask_strategy_name);
		printmsg( stderr, parms_setting );
		exit(-1);
	}
	if (!check_OVERLAPPATTERNCHOOSE(overlapPatternChoose)) {
		fprintf(stderr, "\nError: -overlapPatternChoose=STRING (%s) is not correct!\n", overlapPatternChoose);
		printmsg( stderr, parms_setting );
		exit(-1);
	}
	if (strlen(fn_selectedDatasetsList)==0) {
		fprintf(stderr, "\nError: -selectedDatasetsListFile=FILE is required!\n");
		printmsg( stderr, parms_setting );
		exit(-1);
	}
	if (strcmp(rangeWeight_string,RANGE_WEIGHT_ALL)==0 || strlen(rangeWeight_string)==0)
		init_VEC_FLOAT( &rangeWeight );
	else {
		read_string2VEC_FLOAT_SetFormat( rangeWeight_string, &rangeWeight );
		if ( rangeWeight.n!=2 ) {
			fprintf(stderr, "\nError: -rangeWeight=STRING (%s) is not correct!\n", rangeWeight_string);
			printmsg( stderr, parms_setting );
			exit(-1);
		} else {
			if (rangeWeight.v[0]>rangeWeight.v[1]) {
				fprintf(stderr, "\nError: -rangeWeight=STRING (%s) is not correct!\n", rangeWeight_string);
				printmsg( stderr, parms_setting );
				exit(-1);
			}
		}
	}
	if (strcmp(removeHeavyFrequentEdges_string,REMOVE_HEAVYFREQUENTEDGES_NONE)==0 || strlen(removeHeavyFrequentEdges_string)==0) {
		init_VEC_FLOAT( &removeHeavyFrequentEdges );
	}
	else {
		read_string2VEC_FLOAT_SetFormat( removeHeavyFrequentEdges_string, &removeHeavyFrequentEdges );
		if ( removeHeavyFrequentEdges.n!=2 ) {
			fprintf(stderr, "\nError: -removeHeavyFrequentEdges=STRING (%s) is not correct!\n", removeHeavyFrequentEdges_string);
			printmsg( stderr, parms_setting );
			exit(-1);
		} else {
			if ( removeHeavyFrequentEdges.v[0]<=0 || removeHeavyFrequentEdges.v[0]>1 || removeHeavyFrequentEdges.v[1]<=0 || removeHeavyFrequentEdges.v[1]>1 ) {
				fprintf(stderr, "\nError: -removeHeavyFrequentEdges=STRING (%s) is not correct!\n", removeHeavyFrequentEdges_string);
				printmsg( stderr, parms_setting );
				exit(-1);
			}
		}
	}
	if ((howtoInit_xy!=INIT_XY_BY_ONES) && (howtoInit_xy!=INIT_XY_BY_RAND) && (howtoInit_xy!=INIT_XY_BY_UNIT)) {
		fprintf(stderr, "\nError: -Init_xy=NUM (%d) is not correct!\n", howtoInit_xy);
		printmsg( stderr, parms_setting );
		exit(-1);
	}
	if (useGivenResultFilename && strcmp(fn_results,PREFIX_RESULTFILE_DEFAULT)==0) {
		fprintf(stderr, "\nError: -ResultFile=FILE provide an empty filename!\n");
		printmsg( stderr, parms_setting );
		exit(-1);
	}
}

void init_global_variables()
{
	init_PARMS_SETTING( &parms_setting );
	init_LIST_STRING( MAXCHAR_ID, &selectedNetsID );
	init_LIST_STRING( MAXCHAR, &selectedNetsAnnotation );
	init_VEC_UINT( &selectedNets_INDEX );
	init_TENSOROFNETS( &t );
	init_TENSOROFNETS( &t_backup );
	init_VEC_FLOAT( &rangeWeight );
	init_VEC_FLOAT( &removeHeavyFrequentEdges );
	init_VEC_UINT( &genes_index_excluded );
	init_VEC_UINT( &binary_vec_genes_index_excluded );
}

void free_global_variables()
{
	free_PARMS_SETTING( &parms_setting );
	free_LIST_STRING( &selectedNetsID );
	free_LIST_STRING( &selectedNetsAnnotation );
	free_TENSOROFNETS( &t );
	free_TENSOROFNETS( &t_backup );
	free_VEC_FLOAT( &rangeWeight );
	free_VEC_FLOAT( &removeHeavyFrequentEdges );
	free_VEC_UINT( &genes_index_excluded );
	free_VEC_UINT( &binary_vec_genes_index_excluded );
}

int main(int argc, char *argv[])
{
	char parms[MAXCHAR], settingFile[MAXCHAR], logFile[MAXCHAR], tmpString[MAXCHAR];
	FILE* stream;
	unsigned int nRemovedEdges, ret, i;

	init_global_variables();

	/* Check options of command */
	parse_command_line(argc, argv);

	create_PARMS_SETTING( N_SETTING, &parms_setting );
	if (strlen(fn_parms)==0) { // use default parameters
		assign_PARMS_valueset1( &parms_setting.settings[0] );
		assign_PARMS_valueset2( &parms_setting.settings[1] );
		/*parms_setting.settings[0].a = 0.2;
		parms_setting.settings[0].b = 2;
		parms_setting.settings[0].p = 0.8;
		parms_setting.settings[0].q = 10;
		parms_setting.settings[0].h = 1.2;
		parms_setting.settings[1].a = 0.05;
		parms_setting.settings[1].b = 2;
		parms_setting.settings[1].p = 0.8;
		parms_setting.settings[1].q = 10;
		parms_setting.settings[1].h = 2;*/
	} else { // Two sets of parameters use the same values of parameters defined in the file.
		ret = read_PARMS( &parms_setting.settings[0], fn_parms );
		if (ret==FALSE) { fprintf(stderr, "Terminate accidently.\n"); exit(-1); }
		read_PARMS( &parms_setting.settings[1], fn_parms );
	}

	parms_setting.settings[0].howtoInit_xy = howtoInit_xy;
	parms_setting.settings[1].howtoInit_xy = howtoInit_xy;

	if (strcmp(overlapPatternChoose,OVERLAPPATTERNCHOOSE_NONZEROS)!=0) {
		sprintf( parms, "nEdgeLoad%uminGene%uminNet%uminD%g", nEdgesLoad, minGene, minNet, minDensity );
	} else {
		sprintf( parms, "nEdgeLoad%u", nEdgesLoad );
	}
	if (removeHeavyFrequentEdges.v!=NULL)
		sprintf( parms, "%sFHS%g~%g", parms, removeHeavyFrequentEdges.v[0], removeHeavyFrequentEdges.v[1] );
	if (rangeWeight.v!=NULL)
		sprintf( parms, "%srangeW%g~%g", parms, rangeWeight.v[0], rangeWeight.v[1] );
	sprintf( parms, "%s.basic", parms );

	/* get full files with path */
	fullfile( path_result, fn_results );

	if (!useGivenResultFilename)
		strcat( fn_results, parms );

	read_2COLUMNS_2LIST_STRING( &selectedNetsID, &selectedNetsAnnotation, fn_selectedDatasetsList,DELIMTERS_SELECTEDDATAFILE, MAXCHAR_ID, MAXCHAR );
	if (selectedNetsID.n==0) { fprintf(stderr,"\nError: No graphs are provided in '%s', exit\n",fn_selectedDatasetsList); exit(-1);}
	
	/* print parms used */
	sprintf( settingFile, "%s.SETTING", fn_results );
	if( (stream = fopen( settingFile, "a" )) == NULL ) { errorfile( settingFile, "append" ); exit(0); }
	if ( ! resume ) { /* start a new run */
		print_startmsg( stream ); printparms( stream, parms_setting ); fclose( stream );
		print_startmsg( stdout ); printparms( stdout, parms_setting );
	} else { /* resume the last run */
		print_resumemsg( stream ); printparms( stream, parms_setting ); fclose( stream );
		print_resumemsg( stdout ); printparms( stdout, parms_setting );
	}

	#ifdef DEBUG
		if (debug_nGene<geneTotalNumber) geneTotalNumber = debug_nGene;
		printf("\n--------debug (use first %u nodes and %u graphs)-------\n",geneTotalNumber,selectedNetsID.n);
	#endif

	if (strlen(fn_excluded_genes)>0) {
		read_VEC_UINT( &genes_index_excluded, fn_excluded_genes ); // Note that gene index starts from 1.
		create_VEC_UINT( geneTotalNumber, &binary_vec_genes_index_excluded ); // it is initialized as ZEROs
		for (i=0; i<genes_index_excluded.n; i++) {
			binary_vec_genes_index_excluded.v[genes_index_excluded.v[i]-1] = 1;
		}
		printf("#Nodes_to_be_excluded: %u\n", genes_index_excluded.n);
	}

	if (muleInput==FALSE) {
		#ifdef INPUT_TENSOROFNETS_GENEIDXBY0
			load_nEdges_Of_TENSOROFNETS_GENEIDXBY0( geneTotalNumber, selectedNetsID, nEdgesLoad, rangeWeight, loadUnweighted, &t, PREFIX_DATAFILE, suffixdataFile, path_nets);
		#endif
		#ifdef INPUT_TENSOROFNETS_GENEIDXBY1
			load_nEdges_Of_TENSOROFNETS_GENEIDXBY1( geneTotalNumber, selectedNetsID, nEdgesLoad, rangeWeight, loadUnweighted, &t, PREFIX_DATAFILE, suffixdataFile, path_nets);
		#endif
	} else {
		if (strlen(fn_excluded_genes)>0)
			load_TENSOROFNETS_muleGraphFormat_withExcludedGenes( selectedNetsID, binary_vec_genes_index_excluded, excludeEdgesOfDirectNeighborGenes, &t, PREFIX_DATAFILE, suffixdataFile, path_nets);
		else
			load_TENSOROFNETS_muleGraphFormat( selectedNetsID, &t, PREFIX_DATAFILE, suffixdataFile, path_nets);
	}

	/* print information to indicate all data has been loaded to memory */
	if( (stream = fopen( settingFile, "a" )) == NULL ) { errorfile( settingFile, "append" ); exit(0); }
	print_dataloadmsg( stream ); fclose( stream );
	print_dataloadmsg( stdout );

	/* print edges load information to log file */
	if (!succintOutput) {
		sprintf( logFile, "%s.LOG", fn_results );
		if( (stream = fopen( logFile, "a" )) == NULL ) { errorfile( logFile, "append" ); exit(0); }
		print_nEdges_Of_TENSOROFNETS( t, stream );
		fclose( stream );
	}

	if (maxGene>t.nGene) maxGene = t.nGene; // maxGene is originally assigned as MAXGENE=200
	if (minGene>=t.nGene || minGene>maxGene) {
		fprintf(stderr,"\nError: minNode(%u) should be <#AllNodes(%u) or <=MAXNODE(default %u)\n", minGene, t.nGene, maxGene);
		printmsg(stderr, parms_setting);
		free_global_variables();
		return 0;
	}

	copy_TENSOROFNETS( &t_backup, t );

	LpRelax_byMultipleStrategies( &t, t_backup, parms_setting, nPattern_max, minGene, minNet, minDensity,
		nIteration, nStage, mask_strategy_code, overlapPatternChoose, fn_results, maxGene, resume, removeHeavyFrequentEdges );

	if( (stream = fopen( settingFile, "a" )) == NULL ) { errorfile( settingFile, "append" ); exit(0); }
	print_closingmsg( stream ); fclose( stream );
	print_closingmsg( stdout );

	printf("Free space\n");
	free_global_variables();
	printf("Finish.\n");
	#ifdef SIMULATION_DATA
		getch();
	#endif
	#ifdef DEBUG
		//getch();
	#endif

	return 0;
}
